<?php
/**
 * Osa_chartdata.php - The flash chart data object
 *
 * @author $Author: dtong $
 * @version $Id: Osa_chartdata.php,v 1.8 2012/01/25 10:28:43 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

Class Osa_chartdata {
	public $nodehead=NULL;
	private $nodetail=NULL;
	private $colorindex=0;
	// private $colorcount;
	private $colorgen_obj=NULL;
	public $divid;
	public $title;
	public $count; //Count of total item
	public $url=NULL; //Not used yet

	function __construct() {
		$this->reset();
	}

	/*
	 * Resets the data
	 */
	function reset() {
		$this->divid = 'div_chart'; //Default name, can override
		$this->title = 'Graph';
		$this->nodehead = NULL;
		$this->nodetail = NULL;
		$this->colorindex = 0;
		// $this->colorcount = count($this->colorcodes);
		$this_count = 0;
		if ( $this->colorgen_obj !== NULL )
			$this->colorgen_obj->reset();
	}

	/*
	 * Adds one piece of data, for pie chart, this is one slice
	 */
	function addslice($value='', $label='', $link='', $color='') {
		$label = wordwrap($label, 25);
		if ( $color == '' ) {
			if ( $this->colorgen_obj === NULL ) {
				$this->colorgen_obj = new randomColorGenerator();
			}
				$color = $this->colorgen_obj->getColor();
			}
		$tmp = new Osa_chartnode($value, $label, $link, $color);
		if ( $this->nodehead == NULL ) {
			$this->nodehead = $tmp;
			$this->nodetail = $tmp;
		}
		else {
			$this->nodetail->next = $tmp;
			$this->nodetail = $tmp;
		}
		$this->count++;
	}

	/*
	 * Gets all the values in an array
	 */
	function getvalues() {
		return $this->getarray('value');
	}

	/*
	 * Gets all the colors in an array
	 */
	function getcolors() {
		return $this->getarray('color');
	}

	/*
	 * Gets all the lables in an array
	 */
	function getlables() {
		return $this->getarray('lable');
	}

	/*
	 * Gets all the URL links in an array
	 */
	function getlinks() {
		return $this->getarray('link');
	}

	function getarray($field) {
		$result = array();
		$node = $this->nodehead;
		if ( $node == NULL || !isset($node->{$field}) )
			return $result;
		while ($node != NULL) {
			$result[] = $node->{$field};
			$node = $node->next;
		}
		return $result;
	}
}

/*
 * This is the class for a chart node
 */
Class Osa_chartnode {
	public $color=''; //Color code without the #
	public $value=''; //The pie value
	public $lable=''; //The pie lable
	public $link=''; //The URL link or Jscript code
	public $next=NULL; //Next node in the linked list

	function Osa_chartnode($value, $label='', $link='', $color='') {
		$this->value = $value;
		$this->lable = $label;
		$this->color = $color;
		$this->link = $link;
	}
}

/*
 * This generates a random color. The default is to use the color table first and then use random colors
 * once the table is exhausted. Call getColor(false) to skip the color table.
 */
Class randomColorGenerator {
	private $hexColor = array('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
	private $hexColorCount;
	private $colorBag = array();
	private $colorcodes =
		array('3333FF','66FFCC','990066','666699','FFCC00','FF99CC','339999','9900FF','FF6600','33CCFF',
				'669900','9999FF','FFCC99','FF0000','FF00FF','CC6666','6600CC','CC9999','66CCCC','996600');
	private $colorcodes_override = FALSE;

	function __construct() {
		$this->hexColorCount = count($this->hexColor) - 1;
	}

	// Make sure to call set_colorcodes() again if using custom color table.
	function reset() {
		$this->colorBag = array();
		$this->colorcodes_override = FALSE;
	}

	// To use caller supplied color codes.
	function set_colorcodes($colors) {
		if ( is_array($colors) && count($colors) > 0 ) {
			$this->colorcodes_override = $colors;
		}
	}

	function getColor($use_predefined_colors=TRUE) {
		if ( $use_predefined_colors ) {
			if ( $this->colorcodes_override === FALSE ) {
				$this->colorcodes_override = $this->colorcodes;
			}
			if ( count($this->colorcodes_override) > 0 ) {
				//Using predefined color before getting a random color.
				return array_shift($this->colorcodes_override);
			}
		}
		// Get a random color.
		while ( TRUE ) {
			$newcolor = $this->genRandom() . $this->genRandom() .
									$this->genRandom() . $this->genRandom() .
									$this->genRandom() . $this->genRandom();
			if( !isset($this->colorBag["$newcolor"]) ) {
				$this->colorBag["$newcolor"] = TRUE;
				return $newcolor;
			}
		}
	}

	// Get one random hex number.
	private function genRandom() {
		return $this->hexColor[mt_rand(0, $this->hexColorCount)];
	}
}
