<?php
/**
 * cms_startup - Load startup functions. This code has to be very efficient since all requestes will execute this.
 *
 * @author $Author: dtong $
 * @version $Id: cms_startup.php,v 1.14 2012/03/20 14:38:16 dtong Exp $
 * @copyright Copyright (c) 2011, Tiller Software Co., Ltd.
*/
// Put all system-wide includes here.
require_once(APPPATH . 'libraries/pureobject/cms_exception.php');

class Cms_startup {
	const FILE_EXTENSION = 'todcm';
	const FILE_PATH = FCPATH;
	const CMD_STOP = 'stop';
	const CMD_MAINTENANCE = 'main';
	const CMD_MESSAGE = 'msg';

	// private static $run_functions = array('maintenance_mode_check');
	// All functions should check for null value and init. if necessary
	// This is for performance reason.
	private $CI;
	private $cmd_filename = FALSE;

	function __construct() {
		$this->CI = & get_instance();
		/* foreach (self::$run_functions as $function) {
			$this->$function();
		} */
		$this->maintenance_mode_check();
		// Third-party startup tasks.
		if ( osa_3p_available() ) {
			  $this->CI->cms_3p->load_lib(CMS_3P_LIB_STARTUP, CMS_3P_FOLDER_COMMON);
		}
	}

	/**
	 * File driven site maintenance mode. Files are located at the web root folder.
	 * Also supports setting to maintenance mode from the admid config menu.
	 *
	 * stop.todcm - prevents any one to login including admins
	 * main.todcm - maintenance mode, admins can login
	 * msg.todcm - prints message on login page
	 *
	 * All the 3 files can be prepended with unit_org such as zis_stop.todcm.
	 */
	private function maintenance_mode_check() {
		$site_filename = $this->get_site_filename(); // only defined site.* list
		if ( !$site_filename ) {
			if ($this->CI->config->item('cms_manage_mode')) {
				//This is for checking non-file based maintenance mode. Admin using the config
				//menu to turn maintenance mode on.
				$this->set_maintenance_mode();
			}
			return;
		}
		// must be called before any actual output is sent
		// call this function if has maintenance file
		osa_header_nocache();
		switch ($site_filename):
			 case self::CMD_STOP:
			 	$this->CI->config->set_item('cms_sitestop', TRUE);
			 	$this->set_notice_message();
			 	$this->set_stop_mode();
			 	break;
			 case self::CMD_MAINTENANCE:
			 	$this->CI->config->set_item('cms_manage_mode', TRUE);
			 	$this->set_notice_message();
			 	$this->set_maintenance_mode();
			 	break;
			 case self::CMD_MESSAGE:
			 	$this->set_notice_message();
			 	break;
		endswitch;
	}

	/**
	 * Looks for a command from a list of possible files.
	 * Order of importance: stop.todcm, main.todcm and msg.todcm
	 * File names can also contain org name such as orgname_stop.todcm.
	 * Although file names are treated as case-insensitive but you should always use lower case filenames.
	 *
	 * @return string|false command of site file name
	 */
	private function get_site_filename() {
		// glob function: return array, empty array if no file matched, or FALSE on error
		$dir_list = @glob(self::FILE_PATH . '*.' . self::FILE_EXTENSION);
		if ( empty($dir_list) ) {
			 return FALSE;
		}
		$dir_filelist = array();
		$real_files = array();
		//This loop will get the command(s) from the filelist and save the real file pathname to the classe variable
		foreach ( $dir_list as $full_filepath ) {
			$filename_noextension = strtolower(pathinfo($full_filepath, PATHINFO_FILENAME));
			if ( strpos($filename_noextension, '_') !== FALSE ) {
				if ( osa_str_extract($filename_noextension, '_', TRUE) != strtolower($this->CI->config->item('cms_unit_org'))) {
					continue;
				}
				$cmd = osa_str_extract($filename_noextension, '_', FALSE);
			}
			else {
				$cmd = strtolower($filename_noextension);
			}
			if ( $cmd == self::CMD_STOP ) {
				$this->cmd_filename = $full_filepath;
				return self::CMD_STOP;
			}
			$dir_filelist[] = $cmd;
			$real_files[$cmd] = $full_filepath;
		}
		$sites_array = array(self::CMD_MAINTENANCE, self::CMD_MESSAGE);
		//Loops thru the command array in the order assigned. The order is important.
		//Returns the command if a match is found compare to the command(s) extracted from file list
		foreach ( $sites_array as $sitefile ) {
			if ( in_array($sitefile, $dir_filelist) ) {
				$this->cmd_filename = $real_files[$sitefile];
				return $sitefile;
			}
		}
		return FALSE;
	}

	/**
	 * read string from a site file
	 * @return string message in a file and already line breaks in every newlines
	 */
	private function set_notice_message(){
		if ( !$this->cmd_filename ) {
			return;
		}
		$string_message = file_get_contents( $this->cmd_filename );
		if ( !empty($string_message) ) {
			// '<br />' or '<br>' inserted before all newlines (\r\n, \n\r, \n and \r).
			$string_message = nl2br($string_message);
			return $this->CI->config->set_item('cms_login_message', $string_message);
		}
		else {
			return;
		}
	}

	/**
	 * Kick all users except user who has pass through sesssion
	 * use the same way that maintenance mode do
	 */
	private function set_stop_mode(){
		if ( !$this->check_auth() ) {
			return;
		}
		$login_obj = osa_login_object();
		// no magic password, use maintenance function
		if ( !isset($login_obj->cms_stopmode_passthrough) || empty($login_obj->cms_stopmode_passthrough) ) {
			$this->set_maintenance_mode(TRUE);
		}
		return;
	}
	/**
	 * set system to maintenance mode means clear the current session and redirect to root page
	 * @param bool $kick_admin in stop mode, kick all users including admin (except pass through session)
	 */
	private function set_maintenance_mode($kick_admin = FALSE){
		if ( !$this->check_auth() ) {
			return;
		}
		$login_obj = osa_login_object();
		// login by using magic password
		if ( isset($login_obj->cms_stopmode_passthrough) && $login_obj->cms_stopmode_passthrough === TRUE ) {
	   	return;
		}
		$access_id = $login_obj->accessid;
		// normal user(s) who doesn't has admin role
    	if ( $access_id != CMS_ACCESS_ADMIN || $kick_admin === TRUE ) {
			// using ajax, print or popup the message
			if( osa_is_ajax() ) {
				$msg = $this->CI->config->item('cms_login_message') . '<br />' . lang('gen_maintenance_ajaxmsg');
				echo osa_ajaxmsg_redirecthome($msg);
				$this->CI->login_model->destroyLogin();
				exit;
			}
			// if not ajax, clear session and redirect
			else {
				$this->CI->login_model->destroyLogin();
				redirect();
				exit;
			}
    	}
    	return;
	}

	/**
	 * check authenticated or not
	 * @return boolean
	 */
	private function check_auth(){
		static $auth = NULL;
		if ($auth === NULL) {
			$this->CI->load->model('auth/login_model');
			$login_auth = $this->CI->login_model->isAuthenticated();
			if ( $login_auth ) {
				return $auth = $login_auth;
			}
			// if no login in system, check in survey
			$this->CI->load->model('auth/survey_login_model');
			$survey_auth = $this->CI->survey_login_model->isAuthenticated();
			$auth = $survey_auth;
		}
		return $auth;
	}

}
